"""custom type hints"""
from enum import Enum
from typing import TypedDict, NewType

from PIL.Image import Image
from numpy import ndarray


class DictStrInt(dict[str, int]): pass


class DictStr(dict[str]): pass


class DictStrAny(dict[str, any]): pass


class ListDict(list[dict]): pass


class ListFloat(list[float]): pass


class ListInt(list[int]): pass


class TupleInt(tuple[int]): pass


TupleInt2 = NewType("TupleInt", tuple[int, int])


class TupleNdarrayInt(tuple[ndarray, int]): pass


class TupleNdarrayFloat(tuple[ndarray, float]): pass


class LlistFloat(ListFloat): pass


class TupleFloat(tuple[float]): pass


class TupleFloatAny(tuple[float, any]): pass


PIL_Image = Image


class StrEnum(str, Enum):
    pass


class EmbeddingImage(TypedDict):
    image_embedding: ndarray
    original_size: TupleInt
    transform_matrix: ndarray


class EmbeddingPILImage(TypedDict):
    image_embedding: ndarray
    original_size: TupleInt2
    resized_size: TupleInt2


EmbeddingDict = dict[str, EmbeddingImage]
EmbeddingPILDict = dict[str, EmbeddingPILImage]



"""Get machine learning predictions from geodata raster images"""
from pathlib import Path

from samgis_core.utilities.fastapi_logger import setup_logging


app_logger = setup_logging(debug=True)
PROJECT_ROOT_FOLDER = Path(globals().get("__file__", "./_")).absolute().parent.parent
MODEL_FOLDER = Path(PROJECT_ROOT_FOLDER / "machine_learning_models")



import loguru


format_string = "{time} - {level} - {file} - {function} - ({extra[request_id]}) {message} "


def setup_logging(debug: bool = False, formatter: str = format_string) -> loguru.logger:
    """
    Create a logging instance with log string formatter.

    Args:
        debug: logging debug argument
        formatter: log string formatter

    Returns:
        Logger

    """
    import sys

    logger = loguru.logger
    logger.remove()
    level_logger = "DEBUG" if debug else "INFO"
    logger.add(sys.stdout, format=formatter, level=level_logger)
    logger.info(f"type_logger:{type(logger)}, logger:{logger}.")
    return logger
